@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
- * Copyright (c) 2018-2021 Gavin D. Howard and contributors.
+ * Copyright (c) 2018-2023 Gavin D. Howard and contributors.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -220,20 +220,11 @@ typedef struct BcProgram
 
 #if !BC_ENABLED
 
-/// This define disappears the parameter last because for dc only, last is
-/// always true.
-#define bc_program_copyToVar(p, name, t, last) \
-	bc_program_copyToVar_impl(p, name, t)
-
 /// Returns true if the calculator should pop after printing.
 #define BC_PROGRAM_POP(pop) (pop)
 
 #else // !BC_ENABLED
 
-// This is here to quiet a compiler warning.
-#define bc_program_copyToVar(p, name, t, last) \
-	bc_program_copyToVar_impl(p, name, t, last)
-
 /// Returns true if the calculator should pop after printing.
 #define BC_PROGRAM_POP(pop) (BC_IS_BC || (pop))
 
@@ -320,6 +311,13 @@ bc_program_free(BcProgram* p);
 
 #endif // NDEBUG
 
+/**
+ * Prints a stack trace of the bc functions or dc strings currently executing.
+ * @param p  The program.
+ */
+void
+bc_program_printStackTrace(BcProgram* p);
+
 #if BC_DEBUG_CODE
 #if BC_ENABLED && DC_ENABLED
 
