@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
- * Copyright (c) 2018-2021 Gavin D. Howard and contributors.
+ * Copyright (c) 2018-2023 Gavin D. Howard and contributors.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -1973,7 +1973,7 @@ bc_parse_parse(BcParse* p)
 exit:
 
 	// We need to reset on error.
-	if (BC_ERR(((vm->status && vm->status != BC_STATUS_QUIT) || vm->sig)))
+	if (BC_ERR(((vm->status && vm->status != BC_STATUS_QUIT) || vm->sig != 0)))
 	{
 		bc_parse_reset(p);
 	}
