@@ -2,7 +2,7 @@
 #
 # SPDX-License-Identifier: BSD-2-Clause
 #
-# Copyright (c) 2018-2021 Gavin D. Howard and contributors.
+# Copyright (c) 2018-2023 Gavin D. Howard and contributors.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -37,49 +37,72 @@ testdir=$(dirname "${script}")
 
 outputdir=${BC_TEST_OUTPUT_DIR:-$testdir}
 
-# Command-line processing.
-if [ "$#" -lt 2 ]; then
+# Just print the usage and exit with an error. This can receive a message to
+# print.
+# @param 1  A message to print.
+usage() {
+	if [ $# -eq 1 ]; then
+		printf '%s\n\n' "$1"
+	fi
 	printf 'usage: %s dir script [run_extra_tests] [run_stack_tests] [generate_tests] [time_tests] [exec args...]\n' "$script"
 	exit 1
+}
+
+# Command-line processing.
+if [ "$#" -lt 2 ]; then
+	usage "Not enough arguments; expect 2 arguments"
 fi
 
 d="$1"
 shift
+check_d_arg "$d"
+
+scriptdir="$testdir/$d/scripts"
 
 f="$1"
 shift
+check_file_arg "$scriptdir/$f"
 
 if [ "$#" -gt 0 ]; then
 	run_extra_tests="$1"
 	shift
+	check_bool_arg "$run_extra_tests"
 else
 	run_extra_tests=1
+	check_bool_arg "$run_extra_tests"
 fi
 
 if [ "$#" -gt 0 ]; then
 	run_stack_tests="$1"
 	shift
+	check_bool_arg "$run_stack_tests"
 else
 	run_stack_tests=1
+	check_bool_arg "$run_stack_tests"
 fi
 
 if [ "$#" -gt 0 ]; then
 	generate="$1"
 	shift
+	check_bool_arg "$generate"
 else
 	generate=1
+	check_bool_arg "$generate"
 fi
 
 if [ "$#" -gt 0 ]; then
 	time_tests="$1"
 	shift
+	check_bool_arg "$time_tests"
 else
 	time_tests=0
+	check_bool_arg "$generate"
 fi
 
 if [ "$#" -gt 0 ]; then
 	exe="$1"
 	shift
+	check_exec_arg "$exe"
 else
 	exe="$testdir/../bin/$d"
 fi
@@ -100,8 +123,6 @@ else
 	halt="q"
 fi
 
-scriptdir="$testdir/$d/scripts"
-
 name="${f%.*}"
 
 # We specifically want to skip this because it is handled specially.
